'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.ready = ready;
exports.wrap = wrap;
var INVALID_FUNCTION = 'Invalid callback function!';
var isFunction = function isFunction(fn) {
  return typeof fn === 'function';
};

/**
 * Ready function that accepts a `callback` and optional `options` as params
 * An ready action will be dispatched with provided options
 * before proceeding to the next targeted action
 *
 * @param  {Function} callback
 * @param  {Object}   [options={}]
 * @return {Object}
 */
function ready(callback) {
  var options = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

  return function (_ref) {
    var dispatch = _ref.dispatch,
        getState = _ref.getState;

    var p = new Promise(function (resolve, reject) {
      if (!isFunction(callback)) {
        reject(INVALID_FUNCTION);
      } else {
        resolve(dispatch({ type: 'READY_ACTION', options: options }));
      }
    });

    return p.then(function () {
      return callback(dispatch, getState);
    });
  };
}

/**
 * Wrap function that accepts a `callback` as param
 * NO ready action will be dispatched
 *
 * @param  {Function} callback
 * @return {Object}
 */
function wrap(callback) {
  return function (_ref2) {
    var dispatch = _ref2.dispatch,
        getState = _ref2.getState;
    return new Promise(function (resolve, reject) {
      if (!isFunction(callback)) {
        reject(INVALID_FUNCTION);
      } else {
        resolve(callback(dispatch, getState));
      }
    });
  };
}

/**
 * Redux ready wrapper middleware.
 * Need to be imported as part of application's middlewares for usage.
 *
 * @return {Function|Object}
 */

exports.default = function () {
  return function (store) {
    return function (next) {
      return function (action) {
        if (typeof action === 'function') {
          return action(store);
        }

        return next(action);
      };
    };
  };
};